@@ -14,11 +14,15 @@ from typing import List, NoReturn, Optional, Type, cast
 from pip_audit import __version__
 from pip_audit._audit import AuditOptions, Auditor
 from pip_audit._dependency_source import (
+    PYPI_URL,
     DependencySource,
     PipSource,
+    PyProjectSource,
     RequirementSource,
     ResolveLibResolver,
 )
+from pip_audit._dependency_source.interface import DependencySourceError
+from pip_audit._fix import ResolvedFixVersion, SkippedFixVersion, resolve_fix_versions
 from pip_audit._format import ColumnsFormat, CycloneDxFormat, JsonFormat, VulnerabilityFormat
 from pip_audit._service import OsvService, PyPIService, VulnerabilityService
 from pip_audit._service.interface import ResolvedDependency, SkippedDependency
@@ -134,10 +138,7 @@ def _fatal(msg: str) -> NoReturn:
     sys.exit(1)
 
 
-def audit() -> None:
-    """
-    The primary entrypoint for `pip-audit`.
-    """
+def _parser() -> argparse.ArgumentParser:
     parser = argparse.ArgumentParser(
         prog="pip-audit",
         description="audit the Python environment for dependencies with known vulnerabilities",
@@ -159,6 +160,9 @@ def audit() -> None:
         dest="requirements",
         help="audit the given requirements file; this option can be used multiple times",
     )
+    dep_source_args.add_argument(
+        "project_path", type=Path, nargs="?", help="audit a local Python project at the given path"
+    )
     parser.add_argument(
         "-f",
         "--format",
@@ -183,7 +187,8 @@ def audit() -> None:
         "-d",
         "--dry-run",
         action="store_true",
-        help="collect all dependencies but do not perform the auditing step",
+        help="without `--fix`: collect all dependencies but do not perform the auditing step; "
+        "with `--fix`: perform the auditing step but do not perform any fixes",
     )
     parser.add_argument(
         "-S",
@@ -234,8 +239,63 @@ def audit() -> None:
         help="give more output; this setting overrides the `PIP_AUDIT_LOGLEVEL` variable and is "
         "equivalent to setting it to `debug`",
     )
+    parser.add_argument(
+        "--fix",
+        action="store_true",
+        help="automatically upgrade dependencies with known vulnerabilities",
+    )
+    parser.add_argument(
+        "--require-hashes",
+        action="store_true",
+        help="require a hash to check each requirement against, for repeatable audits; this option "
+        "is implied when any package in a requirements file has a `--hash` option.",
+    )
+    parser.add_argument(
+        "--index-url",
+        type=str,
+        help="base URL of the Python Package Index; this should point to a repository compliant "
+        "with PEP 503 (the simple repository API)",
+        default=PYPI_URL,
+    )
+    parser.add_argument(
+        "--extra-index-url",
+        type=str,
+        action="append",
+        dest="extra_index_urls",
+        default=[],
+        help="extra URLs of package indexes to use in addition to `--index-url`; should follow the "
+        "same rules as `--index-url`",
+    )
+    parser.add_argument(
+        "--skip-editable",
+        action="store_true",
+        help="don't audit packages that are marked as editable",
+    )
+    return parser
+
+
+def _parse_args(parser: argparse.ArgumentParser) -> argparse.Namespace:
+    return parser.parse_args()
+
+
+def _dep_source_from_project_path(project_path: Path, state: AuditState) -> DependencySource:
+    # Check for a `pyproject.toml`
+    pyproject_path = project_path / "pyproject.toml"
+    if pyproject_path.is_file():
+        return PyProjectSource(pyproject_path, ResolveLibResolver(), state)
+
+    # TODO: Checks for setup.py and other project files will go here.
+
+    _fatal(f"couldn't find a supported project file in {project_path}")
+
+
+def audit() -> None:
+    """
+    The primary entrypoint for `pip-audit`.
+    """
+    parser = _parser()
+    args = _parse_args(parser)
 
-    args = parser.parse_args()
     if args.verbose:
         logging.root.setLevel("DEBUG")
 
@@ -245,6 +305,15 @@ def audit() -> None:
     output_desc = args.desc.to_bool(args.format)
     formatter = args.format.to_format(output_desc)
 
+    # Check for flags that are only valid with requirements files
+    if args.requirements is None:
+        if args.require_hashes:
+            parser.error("The --require-hashes flag can only be used with --requirement (-r)")
+        elif args.index_url != PYPI_URL:
+            parser.error("The --index-url flag can only be used with --requirement (-r)")
+        elif args.extra_index_urls:
+            parser.error("The --extra-index-url flag can only be used with --requirement (-r)")
+
     with ExitStack() as stack:
         actors = []
         if args.progress_spinner:
@@ -252,19 +321,39 @@ def audit() -> None:
         state = stack.enter_context(AuditState(members=actors))
 
         source: DependencySource
+        index_urls = [args.index_url] + args.extra_index_urls
         if args.requirements is not None:
             req_files: List[Path] = [Path(req.name) for req in args.requirements]
+            # TODO: This is a leaky abstraction; we should construct the ResolveLibResolver
+            # within the RequirementSource instead of in-line here.
             source = RequirementSource(
-                req_files, ResolveLibResolver(args.timeout, args.cache_dir, state), state
+                req_files,
+                ResolveLibResolver(
+                    index_urls, args.timeout, args.cache_dir, args.skip_editable, state
+                ),
+                require_hashes=args.require_hashes,
+                state=state,
             )
-        else:
-            source = PipSource(local=args.local, paths=args.paths)
+        elif args.project_path is not None:
+            # NOTE: We'll probably want to support --skip-editable here,
+            # once PEP 660 is more widely supported: https://www.python.org/dev/peps/pep-0660/
 
-        auditor = Auditor(service, options=AuditOptions(dry_run=args.dry_run))
+            # Determine which kind of project file exists in the project path
+            source = _dep_source_from_project_path(args.project_path, state)
+        else:
+            source = PipSource(
+                local=args.local, paths=args.paths, skip_editable=args.skip_editable, state=state
+            )
+
+        # `--dry-run` only affects the auditor if `--fix` is also not supplied,
+        # since the combination of `--dry-run` and `--fix` implies that the user
+        # wants to dry-run the "fix" step instead of the "audit" step
+        auditor = Auditor(service, options=AuditOptions(dry_run=args.dry_run and not args.fix))
 
         result = {}
         pkg_count = 0
         vuln_count = 0
+        skip_count = 0
         for (spec, vulns) in auditor.audit(source):
             if spec.is_skipped():
                 spec = cast(SkippedDependency, spec)
@@ -272,6 +361,7 @@ def audit() -> None:
                     _fatal(f"{spec.name}: {spec.skip_reason}")
                 else:
                     state.update_state(f"Skipping {spec.name}: {spec.skip_reason}")
+                skip_count += 1
             else:
                 spec = cast(ResolvedDependency, spec)
                 state.update_state(f"Auditing {spec.name} ({spec.version})")
@@ -280,11 +370,59 @@ def audit() -> None:
                 pkg_count += 1
                 vuln_count += len(vulns)
 
-    # TODO(ww): Refine this: we should always output if our output format is an SBOM
-    # or other manifest format (like the default JSON format).
+        # If the `--fix` flag has been applied, find a set of suitable fix versions and upgrade the
+        # dependencies at the source
+        fixes = list()
+        fixed_pkg_count = 0
+        fixed_vuln_count = 0
+        if args.fix:
+            for fix in resolve_fix_versions(service, result, state):
+                if args.dry_run:
+                    if fix.is_skipped():
+                        fix = cast(SkippedFixVersion, fix)
+                        logger.info(
+                            f"Dry run: would have skipped {fix.dep.name} "
+                            f"upgrade because {fix.skip_reason}"
+                        )
+                    else:
+                        fix = cast(ResolvedFixVersion, fix)
+                        logger.info(
+                            f"Dry run: would have upgraded {fix.dep.name} to " f"{fix.version}"
+                        )
+                    continue
+
+                if not fix.is_skipped():
+                    fix = cast(ResolvedFixVersion, fix)
+                    try:
+                        source.fix(fix)
+                        fixed_pkg_count += 1
+                        fixed_vuln_count += len(result[fix.dep])
+                    except DependencySourceError as dse:
+                        skip_reason = str(dse)
+                        logger.debug(skip_reason)
+                        fix = SkippedFixVersion(fix.dep, skip_reason)
+                fixes.append(fix)
+
     if vuln_count > 0:
-        print(f"Found {vuln_count} known vulnerabilities in {pkg_count} packages", file=sys.stderr)
-        print(formatter.format(result))
-        sys.exit(1)
+        summary_msg = (
+            f"Found {vuln_count} known "
+            f"{'vulnerability' if vuln_count == 1 else 'vulnerabilities'} "
+            f"in {pkg_count} {'package' if pkg_count == 1 else 'packages'}"
+        )
+        if args.fix:
+            summary_msg += (
+                f" and fixed {fixed_vuln_count} "
+                f"{'vulnerability' if fixed_vuln_count == 1 else 'vulnerabilities'} "
+                f"in {fixed_pkg_count} "
+                f"{'package' if fixed_pkg_count == 1 else 'packages'}"
+            )
+        print(summary_msg, file=sys.stderr)
+        print(formatter.format(result, fixes))
+        if pkg_count != fixed_pkg_count:
+            sys.exit(1)
     else:
         print("No known vulnerabilities found", file=sys.stderr)
+        # If our output format is a "manifest" format we always emit it,
+        # even if nothing other than a dependency summary is present.
+        if skip_count > 0 or formatter.is_manifest:
+            print(formatter.format(result, fixes))
