@@ -70,6 +70,30 @@ class PyPIService(VulnerabilityService):
         response_json = response.json()
         results: List[VulnerabilityResult] = []
 
+        # If the dependency has a hash explicitly listed, check it against the PyPI data
+        if spec.hashes:
+            releases = response_json["releases"]
+            release = releases.get(str(spec.version))
+            if release is None:
+                raise ServiceError(
+                    "Could not find release to compare hashes: "
+                    f"{spec.canonical_name} ({spec.version})"
+                )
+            for hash_type, hash_values in spec.hashes.items():
+                for hash_value in hash_values:
+                    found = False
+                    for dist in release:
+                        digests = dist["digests"]
+                        pypi_hash = digests.get(hash_type)
+                        if pypi_hash is not None and pypi_hash == hash_value:
+                            found = True
+                            break
+                    if not found:
+                        raise ServiceError(
+                            f"Mismatched hash for {spec.canonical_name} ({spec.version}): listed "
+                            f"{hash_value} of type {hash_type} could not be found in PyPI releases"
+                        )
+
         vulns = response_json.get("vulnerabilities")
 
         # No `vulnerabilities` key means that there are no vulnerabilities for any version
@@ -86,6 +110,8 @@ class PyPIService(VulnerabilityService):
             # The ranges aren't guaranteed to come in chronological order
             fix_versions.sort()
 
-            results.append(VulnerabilityResult(v["id"], v["details"], fix_versions))
+            results.append(
+                VulnerabilityResult(v["id"], v["details"], fix_versions, set(v["aliases"]))
+            )
 
         return spec, results
