@@ -1,4 +1,5 @@
 import tempfile
+from pathlib import Path
 
 import pytest
 from packaging.version import Version
@@ -11,6 +12,21 @@ from pip_audit._service.interface import (
 )
 
 
+def pytest_addoption(parser):
+    parser.addoption(
+        "--skip-online", action="store_true", help="skip tests that require network connectivity"
+    )
+
+
+def pytest_runtest_setup(item):
+    if "online" in item.keywords and item.config.getoption("--skip-online"):
+        pytest.skip("skipping test that requires network connectivity due to `--skip-online` flag")
+
+
+def pytest_configure(config):
+    config.addinivalue_line("markers", "online: mark test as requiring network connectivity")
+
+
 @pytest.fixture(autouse=True)
 def spec():
     def _spec(version):
@@ -34,6 +50,7 @@ def vuln_service():
                         id="fake-id",
                         description="this is not a real result",
                         fix_versions=[fixed],
+                        aliases=set(),
                     )
                 ]
 
@@ -48,6 +65,9 @@ def dep_source(spec):
         def collect(self):
             yield spec("1.0.1")
 
+        def fix(self, _) -> None:
+            raise NotImplementedError
+
     return Source
 
 
@@ -56,3 +76,16 @@ def cache_dir():
     cache = tempfile.TemporaryDirectory()
     yield cache.name
     cache.cleanup()
+
+
+@pytest.fixture
+def req_file():
+    def _req_file():
+        req_file = tempfile.NamedTemporaryFile()
+        req_file.close()
+
+        req_path = Path(req_file.name)
+        assert not req_path.exists()
+        return req_path
+
+    return _req_file
